$TITLE(Assembly example program)
$DEBUG 

;Hours and minutes will be displayed on LCD display
;Dot between hours and minutes will blink
 
$INCLUDE(C:\APPLIC\I2C\VAR_DEF.ASM)     ;Include I2C var. definitions
$INCLUDE(C:\APPLIC\I2C\I2C.MAC)         ;Include I2C macro's

CLOCK_ADR   EQU 0A2h                    ;Address of PCF8583
CL_SUB_ADR  EQU 01h                     ;Sub address for reading time
LCD_ADR     EQU 74h                     ;Address of PCF8577

RAMVAR  SEGMENT DATA                    ;Segment for variables
USER    SEGMENT CODE                    ;Segment for application program

        RSEG RAMVAR
STACK:      DS 10                       ;Stack area
TIME_BUFFER:DS 4                        ;Buffer for I2C strings
LCD_BUFFER: DS 5

        CSEG AT 00
        LJMP APL_START


        RSEG USER
APL_START:
        MOV DPTR,#LCD_TAB               ;Pointer to segment table
        MOV SP,#STACK-1                 ;Initialise stack
        MOV LCD_BUFFER,#00              ;Control word for LCD driver
        %INIT_IIC(22h,TIME_BUFFER)      ;Initialise I2C interface
        CLR A                           ;Prepare buffer for clock int.
        MOV TIME_BUFFER,A
        MOV TIME_BUFFER+1,A
        %IIC_WRITE(CLOCK_ADR,2,TIME_BUFFER) ;Initialise clock

REPEAT: %IIC_READ_SUB(CLOCK_ADR,4,TIME_BUFFER,CL_SUB_ADR) ;Read time       

;Time has been read. Order: hundreds of sec's, sec's, min's and hr's
        MOV A,TIME_BUFFER+3             ;Mask of hour counter
        ANL A,#3Fh
        MOV TIME_BUFFER+3,A

        CALL CONVERT                    ;Convert time data to LCD segment data

;Check if dot has to be switched on
        ORL LCD_BUFFER+3,#01h
;If lsb of seconds is '0', then switch on dp
        MOV A,TIME_BUFFER+1             ;Get seconds
        RRC A
        JC PROCEED                     
        ORL LCD_BUFFER+1,#01            ;Switch on dp

;Display new time
PROCEED: %IIC_WRITE(LCD_ADR,5,LCD_BUFFER)
        SJMP REPEAT                     ;Read new time


;CONVERT converts BCD data of time to segment data
CONVERT:MOV R0,#LCD_BUFFER+1            ;R0 is pointer
        MOV A,TIME_BUFFER+3             ;Get hours
        SWAP A                          ;Swap nibbles
        CALL LCD_DATA                   ;Convert 10's of hours
        MOV A,TIME_BUFFER+3
        CALL LCD_DATA                   ;Convert hours
        MOV A,TIME_BUFFER+2             ;Get minutes
        SWAP A
        CALL LCD_DATA                   ;Convert 10's of minutes
        MOV A,TIME_BUFFER+2
        CALL LCD_DATA                   ;Convert minutes
        RET

;LCD_DATA gets data from segment table and stores it in LCD_BUFFER
LCD_DATA:
        ANL A,#0FH                      ;Mask off LS-nibble
        MOVC A,@A+DPTR                  ;Get segment data
        MOV @R0,A                       ;Save segment data
        INC R0
        RET
;
;Conversion table for LCD
LCD_TAB:
        DB 0FCH,60H,0DAH                ;'0','1','2'
        DB 0F2H,66H,0B6H                ;'3','4','5'
        DB 3EH,0E0H,0FEH                ;'6','7','8'
        DB 0E6H                         ;'9'
;
        END


















