$TITLE(Error checking)
$DEBUG
$NOLIST
$INCLUDE(C:\TOOLS\TASKING\ASM51\INCLUDE\8051\REG528.INC)
$LIST
;
;ERR_CHK does error checking when a bit is received/transmitted
;
;On entry Carry contains bit that has been send when in master-transmitter mode
;
;Checks: * SCL is stretched. If no time-out,then no error
;          condition.
;        * SCL still low. No error condition
;        * Time-out
;        * Spurious START/STOP conditions
;        * Arbitration lost
;
;******************************************************************************
;
                I2C_DRIVER SEGMENT CODE INBLOCK
                RSEG I2C_DRIVER

                EXTRN DATA(STATUS,SLAVE_ADR,COUNT_1,COUNT_2)
                EXTRN DATA(SUB_ADR,SOURCE_PTR_1,SOURCE_PTR_2)
                EXTRN BIT(IIC_ERROR,NO_ACCESS,TIME_ERR,ARB_LOST,BUS_ERROR)
                EXTRN BIT(EN_STOP,EN_ARB)
;
;******************************************************************************


                PUBLIC  ERROR_CHK

                TEMP            SET R1
                BIT_CNT         SET R2
                SOFT_TIM        SET R3

ERROR_CHK:
        XCH A,TEMP              ;Store ACC. Contains trx/rcv byte
        JNB CLH,BUS_STR         ;CLH=0; SCL is stretched
        JB SCI,EXIT             ;SCL still high. Wait till finished.
        JNB BB,BUS_ERR          ;Spurious STOP condition detected
        MOV A,S1INT
        JNB ACC.7,NXT_CHK       ;Spurious START condition detected 
BUS_ERR:SETB BUS_ERROR          ;Spurious START/STOP is bus-error
        SJMP EX_ERR

NXT_CHK:JNB EN_ARB,EXIT         ;If EN_ARB was '0' no ARB_LOST check
                                ;ARB_LOST if SDA=0 and 'send bit'=1
        JB SDI,EXIT
        JNC EXIT
        SETB ARB_LOST           ;Arbitration lost
        SETB SDO                ;Release data line
        SJMP EX_ERR             ;Do not release SCL line when exit. 
                                ;ARB_LOST may be during sending of 
                                ;slave address

BUS_STR:DJNZ SOFT_TIM,EXIT      ;SCL stretched, no time-out
        SETB TIME_ERR           ;SCL stretched, time-out 

EX_ERR: SETB IIC_ERROR
EXIT:   XCH A,TEMP              ;Restore accu
        RET
;
$EJECT
;

;******************************************************************************
;EXCHANGE copies the following parameters:
; SUB_ADR      --> SLAVE_ADR
; COUNT_2      --> COUNT_1
; SOURCE_PTR_1 --> SOURCE_PTR_2
;******************************************************************************

                PUBLIC EXCHANGE,SUB_EXCH

EXCHANGE:
        MOV SLAVE_ADR,SUB_ADR
SUB_EXCH:
        MOV COUNT_1,COUNT_2
        MOV SOURCE_PTR_1,SOURCE_PTR_2
        RET
;
;
;******************************************************************************
;CONTEXT_SW saves PSW and copies I2C_PSW to PSW
;******************************************************************************

                PUBLIC CONTEXT_SW

                CONTEXT SET R6
                
                USING 1

CONTEXT_SW:
        MOV A,PSW               ;Save current PSW
        MOV PSW,#08             ;Select RB1 
        MOV CONTEXT,A
        RET
;
;
        END
