$TITLE(IIC_Read command/IIC_Read_Status command)
$DEBUG
$NOLIST
$INCLUDE(C:\TOOLS\TASKING\ASM51\INCLUDE\8051\REG528.INC)
$LIST
;
;I2C master receive routine.
;IIC_Read_Status only reads 1 byte
;
;PL/M declaration: 
;       IIC_Read:  Procedure(Sl_Address,Nr_Bytes,Source_ptr) Byte|Bit Ext;
;                  Declare(Sl_Address,Nr_bytes,Source_ptr) Byte;      
;                  End;
;
;Input parameters: Sl_Address: Address of slave transmitter 
;                  Nr_Bytes:   Number of bytes to receive (excl. slave address)
;                  Source_ptr: Pointer to receive buffer (IDATA 0..255)
;Output parameter: Contents of STATUS is passed to PL/M
;
;
;       IIC_Read_Status: Procedure(Sl_Address,Dest_ptr) Byte|Bit Ext;
;                        Declare(Sl_Address,Dest_ptr) Byte|Bit Ext;
;                        End;
;
;Used registers:   R0: Pointer to receive buffer
;                  R2: Bit counter
;                  R3: Time_out register
;                  R4: Byte counter
;
;******************************************************************************

                PUBLIC _IIC_READ,_IIC_READ_STATUS
                PUBLIC RCV_LOOP,RCV_NO_STOP

                EXTRN  CODE(START_RCV,I2C_STOP,ERROR_CHK,RCV_BY,CONTEXT_SW)
                EXTRN  DATA(COUNT_1,SOURCE_PTR_1)
                EXTRN  BIT(IIC_ERROR,EN_STOP)


                I2C_DRIVER SEGMENT CODE INBLOCK
                RSEG I2C_DRIVER

                BUF_POINT       SET R0
                BIT_CNT         SET R2
                SOFT_TIM        SET R3
                BYTE_CNT        SET R4

;******************************************************************************
;Receive bytes in buffer to I2C bus
;******************************************************************************
_IIC_READ_STATUS:
        MOV COUNT_1,#01         ;Init count_1 for read_status
_IIC_READ:
        SETB EN_STOP            ;Enable STOP generation
        ACALL CONTEXT_SW        ;Select RB1
RCV_NO_STOP:
        ACALL START_RCV         ;Initialise master and generate START
        JB IIC_ERROR,EXIT       ;Test if error occurred
RCV_LOOP:
        MOV BUF_POINT,SOURCE_PTR_1;Init buffer pointer
        MOV BYTE_CNT,COUNT_1    ;Init byte counter
NXT_RCV:ACALL RCV_BY            ;Receive byte and update BUF_POINT
        JB IIC_ERROR,EXIT       ;Test if error occurred
        DJNZ BYTE_CNT,NXT_RCV   ;Check if more bytes have to be received
EXIT:   ACALL I2C_STOP          ;Finish I2C reception
        RET
;
        END
