$TITLE(IIC_Write_Memory)
$DEBUG
$NOLIST
$INCLUDE(C:\TOOLS\TASKING\ASM51\INCLUDE\8051\REG528.INC)
$LIST
;
;I2C master transmit routine with software auto increment of
;sub address and delay of 40ms between sequences.
;
;PL/M declaration: 
;IIC_Write_Memory: Procedure(Sl_Address,Count_1,Source_ptr,Sub_Address)
;                                                    Bit|Byte External;
;                  Declare(Sl_Address,Count_1,Source_ptr,Sub_Address) Byte;
;                  End;
;
;Input parameters: Sl_Address: Address of slave receiver
;                  Count_1:    Number of bytes to transmit
;                              (excl. slave address and sub address)
;                  Source_ptr: Pointer to send buffer (IDATA 0..255)
;                  Sub_Address:Sub address with slave device
;Output parameter: STATUS byte and IIC_Error bit.
;
;Used registers: see _IIC_Write_Sub
;                R2: Delay counter
;                R3: Delay counter
;                R5: Contains number of sequence to be transmitted
;
;
;******************************************************************************

                PUBLIC _IIC_WRITE_MEMORY
                
                EXTRN CODE(_IIC_WRITE_SUB,CONTEXT_SW)
                EXTRN DATA(SUB_ADR,COUNT_1,COUNT_2,SOURCE_PTR_1,SOURCE_PTR_2)
                EXTRN BIT(IIC_ERROR)

                I2C_DRIVER SEGMENT CODE INBLOCK 
                RSEG I2C_DRIVER

;******************************************************************************
;
_IIC_WRITE_MEMORY:
        MOV COUNT_2,COUNT_1     ;Get number of sequences
        MOV COUNT_1,#01         ;Sequence contains 1 data byte
NXT_TRX:ACALL _IIC_WRITE_SUB    ;Transmit sequence with STOP
        JB IIC_ERROR,EXIT       ;Error checking and STOP done in _IIC_WRITE

        MOV A,#202D             ;Execute delay of 40ms (16MHz Xtal)
DEL_2:  MOV SOURCE_PTR_2,#130D  ;SOURCE_PTR_2 and ACC used as delay registers
DEL_1:  DJNZ SOURCE_PTR_2,DEL_1
        DEC A
        JNZ DEL_2

NXT_SEQ:INC SUB_ADR             ;Increment sub address for next sequence
        INC SOURCE_PTR_1        ;Increment pointer to next byte
        DJNZ COUNT_2,NXT_TRX    ;Transmit next sequence
EXIT:   RET
;
        END
