$TITLE(Slave interrupt routine)
$DEBUG
$NOLIST
$INCLUDE(C:\TOOLS\TASKING\ASM51\INCLUDE\8051\REG528.INC)
$LIST
;
;This routine handles I2C interrupts.
;8xC528 I2C interface enters in slave mode.
;After testing R/W bit, 8xC528 will go in slave-transmit or
;slave-receive mode.
;Source or destination buffer for data uses pointer SLAVE_SUB_ADDRESS
;Slave routine will use register bank 01
;
;******************************************************************************
;Interrupt entry point

                CSEG AT 53H

                LJMP __PIP0A    ;Vector to interrupt handler
;
;******************************************************************************

                I2C_DRIVER SEGMENT CODE INBLOCK
                RSEG I2C_DRIVER

                PUBLIC MST_ENTRY
                EXTRN  DATA(SLAVE_SUB_ADDRESS)
                EXTRN  BIT(ARB_LOST)

                BUF_POINT SET R0
                OWN_SLAVE SET R7
                BIT_CNT   SET R2

;******************************************************************************

__PIP0A:PUSH ACC        ;Push CPU status on stack
        PUSH PSW
        MOV PSW,#08H    ;Select registerbank 01

;******************************************************************************
;Check slave address
;******************************************************************************

        ORL S1SCS,#01000010B ;Disable SCL generation and enable SCL stretching
        ACALL RCV_SL_BY ;Receive slave address, on exit SCL is stretched
PROC:   MOV C,ACC.0     ;Store R/W bit in F0
        MOV F0,C
        XRL A,OWN_SLAVE ;Compare received slave address
        CLR ACC.0       ;Ignore R/W bit
        JNZ NO_MATCH    ;Leave slave-routine if there is no match
        CLR C           ;Send ACK
        ACALL SEND_ACK
        MOV BUF_POINT,SLAVE_SUB_ADDRESS  ;Get buffer pointer
        MOV C,F0        ;Restore R/W bit
        JNC RCV_SLAVE   ;Test R/W bit


;******************************************************************************
;Slave transmitter mode
;******************************************************************************


NXT_TRX:MOV A,@BUF_POINT;Get byte to send
        MOV BIT_CNT,#08 ;Init bit counter

NXT_TRX_BIT:
        MOV S1BIT,A     ;Trx bit and stretch after transmission
        RL A            ;Prepare next bit to send
        JNB WBF,$       ;Test if bit is sent
        DJNZ BIT_CNT,NXT_TRX_BIT ;Test if all bits are sent

        SETB SDO        ;Release SDA line for NO_ACK/ACK reception
        MOV A,S1BIT     ;Stop stretching
        JNB RBF,$       ;Test is ACK bit is received
        MOV C,SDI       ;Read bit, SCL remains stretched
        JC EXIT         ;NO_ACK received. Exit slave routine
        INC BUF_POINT   ;ACK received. Update pointer for next byte to send
        SJMP NXT_TRX
        
;******************************************************************************
;Slave receiver mode
;******************************************************************************

RCV_SLAVE:              ;Entry in slave-receiver mode
        ACALL RCV_SL_BY ;Receive byte
        JNC EXIT        ;If STOP is detected, then exit
        MOV @BUF_POINT,A;Store received byte
        CLR C           ;Send ACK
        CALL SEND_ACK
        INC BUF_POINT   ;Update pointer
        SJMP RCV_SLAVE  ;Receive next byte


;******************************************************************************
;Receive byte routine
;On exit, received byte in accu
;On exit CY=0 if STOP is detected
;******************************************************************************
RCV_SL_BY:
        MOV BIT_CNT,#08
        MOV A,S1BIT     ;Disable stretching from START or previous ACK
        CLR A
RCV_BIT:
        JNB BB,STOP_RCV ;Test if STOP-condition is received
        JNB RBF,RCV_BIT ;Wait till received bit is valid
        CJNE BIT_CNT,#01,ASSEM_BIT ;Check if last bit is to be received

        MOV C,SDI       ;Get last bit without stopping stretching
        RLC A
        SETB C          ;No STOP detected
        RET

ASSEM_BIT:
        ORL A,S1BIT     ;Receive bit; release RBF,CLH and SCL stretching
        RL A
        DJNZ BIT_CNT,RCV_BIT

STOP_RCV: 
        CLR C           ;STOP detected
        RET

;******************************************************************************
;Send ACK/NO_ACK. Value of ACK in Carry
;******************************************************************************
SEND_ACK:
        RRC A
        MOV S1BIT,A     ;Carry to SDA line
        JNB WBF,$       ;Test if ACK/NO_ACK is sent
        SETB SDO        ;Release SDA line
        RET

;*****************************************************************************
;No match between received slave-address and own-slave-address
;*****************************************************************************
NO_MATCH:
        SETB C          ;Send NO_ACK
        ACALL SEND_ACK
        SJMP EXIT

;****************************************************************************
;Entry point when an arbitration-lost condition is detected in master-mode.
;****************************************************************************
MST_ENTRY:
        RL A            ;Restore slave address  sofar
        CLR ACC.0
        ORL S1SCS,#01000010B ;Disable SCL generation and enable SCL stretching
        ACALL RCV_BIT   ;Proceed with receiving rest of slave address
        SJMP PROC


;******************************************************************************
;Exit from interrupt routine
;******************************************************************************

EXIT:   CLR STR         ;Disable stretching on next falling SCL edges
        MOV A,S1BIT     ;Stop current SCL stretching
        JNB ARB_LOST,EX_SL
        RET             ;Exit when entered from master mode
EX_SL:  POP PSW         ;Restore old CPU status
        POP ACC
        RETI

        END
