$TITLE(Generate START and trx slave-address)
$DEBUG
$NOLIST
$INCLUDE(C:\TOOLS\TASKING\ASM51\INCLUDE\8051\REG528.INC)
$LIST
;
;
; I2C_START checks if the bus is free and then generates a START condition
; After a time-out, the no-access and IIC_Error flag will be set.
; Time_out determined by BYTE_3 of PL/M-call
; After START the slave-address and R/W bit will be send.
; GEN_START generates a START condition without checking the bus.
; 
;******************************************************************************

                I2C_DRIVER SEGMENT CODE INBLOCK
                RSEG I2C_DRIVER

                EXTRN CODE(TRX_BY,MST_ENTRY)
                EXTRN DATA(STATUS,SLAVE_ADR)
                EXTRN BIT(IIC_ERROR,NO_ACCESS,ARB_LOST,BUS_ERROR)

;******************************************************************************

                PUBLIC GEN_START_RCV,GEN_START_TRX
                PUBLIC START_RCV,START_TRX

                CNT_BUFF      SET R2      
                CNT_BUFF_1    SET R0

GEN_START_RCV:
        MOV A,SLAVE_ADR         ;Entry for rcv repeat START
        ORL A,#01
        SJMP REP_START
GEN_START_TRX:
        MOV A,SLAVE_ADR         ;Entry for trx repeat START
        ANL A,0FEH
REP_START:
        SETB SDO                ;Release SDA and SCL for START generation
        NOP
        SETB SCO
        MOV CNT_BUFF,#0FFH      ;Init time out counter
NEW_TEST:
        JB CLH,START_OK         ;Check if rising edge on SCL is detected
        DJNZ CNT_BUFF,NEW_TEST
        SETB BUS_ERROR          ;No rising edge on SCL possible
        SJMP ERROR
START_OK:
        MOV CNT_BUFF,#02H       ;Delay for t(BUFF)
        DJNZ CNT_BUFF,$
        SJMP GEN_START

START_RCV:
        MOV A,SLAVE_ADR         ;Entry for rcv START
        ORL A,#01
        SJMP I2C_START
START_TRX:
        MOV A,SLAVE_ADR         ;Entry for trx START
        ANL A,#0FEH
I2C_START:
        SETB EA
        CLR ES1                 ;Clear I2C interrupt
        ANL STATUS,#0C0H        ;Clear RD bits of STATUS register

        MOV CNT_BUFF_1,#100     ;Initialise time_out counters
NEW_SEQ:MOV CNT_BUFF,#0FFH 
RETRY:  JNB BB,DIS_INT          ;Test if bus is busy. If not, clear EA before START
        DJNZ CNT_BUFF,RETRY     ;Bus busy; update time-out and do new test
        DJNZ CNT_BUFF_1,NEW_SEQ
ACCESS_ERR:
        SETB NO_ACCESS          ;Set error flags
ERROR:  SETB IIC_ERROR
        RET

DIS_INT:CLR EA                  ;Clear interrupts. Ext. interrupt may appear
        JB BB,I2C_START         ;During that time an external master may
                                ;have accessed the bus.
GEN_START:
        CLR SDO
        MOV CNT_BUFF,#04        ;Delay for t(HD;STA): 10 cycles
        DJNZ CNT_BUFF,$
        CLR SCO                 ;Clear SCL (Also enable auto-clock mode)
        SETB EA                 ;Enable interrupts again
        ACALL TRX_BY            ;Transmit slave address
        JNB ARB_LOST,EXIT     
        ACALL MST_ENTRY         ;When arbitration lost, then go to slave
                                ;receiver mode.
EXIT:   RET
;
        END
