$TITLE(Routines generates a STOP condition)
$DEBUG
$NOLIST
$INCLUDE(C:\TOOLS\TASKING\ASM51\INCLUDE\8051\REG528.INC)
$LIST
;
;I2C_STOP finishes a I2C trx/rcv sequence.
;A STOP condition will be generated except when the 'arbritration lost' flag
;is set or when EN_STOP is '0'.
;When BUS_ERROR or TIME_ERR are set, pending interrupt and stretching are
;cleared (spurious START), and 9 clocks are generated via software (SDA=1) to
;release the bus from a hanging slave-transmitter. 1 of the 9 clocks will be a
;NO_ACK for the slave transmitter.
;A check is done to see if both SDA and ACL are '1', when finishing the STOP
;condition.
;When leaving the routine: ACC = STATUS
;                          CY  = IIC_ERROR
;                          I2C interrupt enabled
;
;******************************************************************************

                I2C_DRIVER SEGMENT CODE INBLOCK
                RSEG I2C_DRIVER

                EXTRN DATA(STATUS)
                EXTRN BIT(IIC_ERROR,NO_ACCESS,TIME_ERR,ARB_LOST,BUS_ERROR)
                EXTRN BIT(EN_STOP,EN_ARB)
;******************************************************************************

                PUBLIC I2C_STOP

                DELAY    SET R0
                SET_UP   SET R2
                CONTEXT  SET R6

I2C_STOP:
        JB IIC_ERROR,NXT_CHK    ;Ignore EN_STOP if IIC-error has occurred
        JNB EN_STOP,EXIT        ;Check EN_STOP,ARB_LOST and NO_ACCESS for
NXT_CHK:JB ARB_LOST,ARB_UPD     ;STOP generation
        JB NO_ACCESS,UPDATE
        JB TIME_ERR,FREE_BUS
        JNB BUS_ERROR,PREP_STOP
                                ;BUS_ERROR=1
        SETB SCO                ;Possible spurious START. Release auto-clock
        MOV A,S1BIT             ;Disable possible stretching and clear S1INT 
        ACALL GEN_CLK           ;Finish SCL cycle, SCL will be '0'

FREE_BUS:                       ;Generate 9 clocks (BUS_ERROR or TIME_ERR is '1')
        SETB SDO                ;Release SDA
        MOV SET_UP,#09
NXT_CLK:ACALL GEN_CLK           ;Generate SCL cycle
        DJNZ SET_UP,NXT_CLK
                                
PREP_STOP:
        CLR SDO                 ;Clear SDA 
        NOP
        SETB SCO                ;Set SCL

        MOV DELAY,#100          ;Set stretch time-out 
DEL:    MOV SET_UP,#0FFH
NEW_TEST:
        JB SCI,GEN_STOP         ;Check if SCL gets '1', may be stretched
        DJNZ SET_UP,NEW_TEST      
        DJNZ DELAY,DEL

        SJMP STOP_F             ;Time out error
GEN_STOP:
        MOV SET_UP,#03H         ;STOP set_up time; 7 machine cycles
        DJNZ SET_UP,$           ;t(SU;STO) delay
        SETB SDO                ;Set SDA
        NOP                     ;Wait for edge to finish. This is in 
        NOP                     ;tBUF
        JB BB,STOP_F            ;If Bus Busy then error,not possible to 
                                ;generate STOP
        SJMP UPDATE             ;Update return parameters

STOP_F: SETB IIC_ERROR          ;Set error flags
        SETB BUS_ERROR
                                ;Exit point for I2C routines
ARB_UPD:SETB SCO                ;Release SCL when arbitration lost
UPDATE: MOV A,STATUS            ;Return STATUS and IIC_ERROR
        ANL A,#03FH             ;Mask off WR-bits of STATUS
        MOV PSW,CONTEXT         ;Restore context
        MOV C,IIC_ERROR
        SETB CLH                ;Set CLH,RBF and WBF
        MOV S1INT,#00           ;Clear pending I2C interrupt of spurious START
        SETB ES1                ;Enable I2C interrupt
EXIT:   RET
;
;
GEN_CLK:MOV DELAY,#04           ;Generate 1 SCL cycle
        SETB SCO
        DJNZ DELAY,$
        MOV DELAY,#04
        CLR SCO
        DJNZ DELAY,$
        RET

        END
